import { ListenerRegistry } from "./util/ListenerRegistry";
import { WithRequiredKeys } from "./util/types";
import { ComponentName, IEntity } from "./World";
export declare type Query<T extends IEntity> = ComponentName<T>[];
export declare type QueriedEntity<TEntity extends IEntity, TQuery extends Query<TEntity>> = WithRequiredKeys<TEntity, TQuery[number]> & TEntity;
export declare class Archetype<TEntity extends IEntity, TQuery extends Query<TEntity> = Query<TEntity>> {
    query: TQuery;
    /** A list of entities belonging to this archetype. */
    entities: QueriedEntity<TEntity, TQuery>[];
    /** Listeners on this event are invoked when an entity is added to this archetype's index. */
    onEntityAdded: ListenerRegistry<(entity: TEntity) => void>;
    /** Listeners on this event are invoked when an entity is removed from this archetype's index. */
    onEntityRemoved: ListenerRegistry<(entity: TEntity) => void>;
    constructor(query: TQuery);
    indexEntity(entity: TEntity): void;
    removeEntity(entity: TEntity): void;
}
